'''
Created on Feb 22, 2019

@author: reynolds
'''

import struct, os, sys

from wrtdk.parser.parser import parser
from wrtdk.parser.msg.wrtmsg import udp_wrapper

class lidar_lite_v3(parser):
    ''' classdocs '''

    def __init__(self):
        ''' Constructor '''
        super().__init__()
        
        self.M_2_CM = 100.0
        self.CM_2_M = 1.0/100.0
        
        self.reset()
        
    def reset(self):
        ''' resets the parser '''
        self._health = -1
        self._regulation = -1
        self._peakdet = -1
        self._ref_overflow = -1
        self._sig_overflow = -1
        self._busy = -1
        self._strength = self._nan()
        self._d = self._nan()
        
    def get(self,health=0,reg=0,
            roverflow=0,soverflow=0,
            dpeak=0,busy=0,strength=0,d=0):
        return struct.pack('>BBH',
                           ((busy & 0x01) | 
                           (soverflow & 0x01) << 1 | 
                           (roverflow & 0x01) << 2 | 
                           (dpeak & 0x01) << 3 | 
                           (reg & 0x01) << 4 | 
                           (health & 0x01) << 5) & 0xff,
                           strength & 0xff,
                           int(d*self.M_2_CM))
        
    def parse(self,msg):
        ''' parses the message '''
        self.reset()
        
        try:
            b,self._strength,self._d = struct.unpack('>BBH',msg)
            self._d *= self.CM_2_M
            self._health       = (b & 0b00100000) >> 5
            self._regulation   = (b & 0b00010000) >> 4
            self._peakdet      = (b & 0b00001000) >> 3
            self._ref_overflow = (b & 0b00000100) >> 2
            self._sig_overflow = (b & 0b00000010) >> 1
            self._busy         = (b & 0b00000001)
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data '''
        return [self._health,self._regulation,self._peakdet,
                self._ref_overflow,self._sig_overflow,self._busy,
                self._strength,self._d]


'''
Created on Dec 4, 2019

@author: Reynolds
'''
import unittest
from wrtdk.parser.tfmag.qmag import qmag


class Test(unittest.TestCase):


    def setUp(self):
        self.tf = 50e3
        self.strength = 10
        self.qmag = qmag()

    def test_get(self):
        b = self.qmag.get(self.tf,self.strength)
        self.qmag.parse(b)
        self.assertFalse(self.qmag.hasErrored())
        d = self.qmag.getData()
        self.assertAlmostEqual(self.tf,d[0][0],1)
        self.assertEqual(self.strength,d[1][0])
        
    def test_parse(self):
        msg = b'!326611627@77'
        matlab_ans = 54350.6458794415521
        self.qmag.parse(msg)
        self.assertFalse(self.qmag.hasErrored())
        d = self.qmag.getData()
        
        self.assertAlmostEqual(matlab_ans,d[0][0],10)

if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()


'''
Created on Feb 20, 2019

@author: reynolds
'''

from wrtdk.io.sim.simulator import simulator
from wrtdk.parser.msg.wrtmsg import udp_wrapper

class sim_simulator(simulator):
    ''' classdocs '''

    def __init__(self):
        ''' Constructor '''
        super().__init__()    
        
    def read(self,filename,filt=None):
        simulator.read(self, filename)
        
        if not self.didError():
            # initialize the message parser and variable to store the bytes
            h = udp_wrapper()
            b = None
            pos = 0
            t = 0
            
            # open the file
            with open(filename,'rb') as f:
                b = f.read()# read in the bytes
                blen = len(b)# get the length of the buffer
                
                while pos < blen:
                    h.parse(b[pos:pos+h.LENGTH])
                    if h.hasErrored():
                        pos += 1
                        print('Error reading header:',h.hasErrored())
                        continue
                    
                    if filt is not None:
                        if h.getType() in filt:
                            self._msgs.append(b[pos:h.getMsgEnd(pos)])
                            self._dt.append(min(0.01,abs(h.getMCU()-t)))
                            t = h.getMCU()
                            
                        pos += h.getMsgEnd()
                    else:
                        self._msgs.append(b[pos:h.getMsgEnd(pos)])
                        self._dt.append(min(0.01,abs(h.getMCU()-t)))
                        t = h.getMCU()
                        pos += h.getMsgEnd()
                
                # re-adjust the change in time
                self._dt = self._dt[1::] + [4]
                
            if b is not None:
                print('Done')
                self._len = len(self._msgs)
                return True
            else:
                print('Error')
                return False
        return False
        
def test():
    ''' tests the simulator '''
    #port = socket.socket(socket.AF_INET,socket.SOCK_DGRAM)
    s = sim_simulator()
    s.read(r'C:\Users\reynolds\Documents\1704_madunit\data\20190227_imu-vmag_compare\imu_vcmag_test_1.dat')
    
if __name__ == '__main__':
    test()


'''
Created on Dec 17, 2019

@author: Reynolds
'''

import numpy as np

from wrtdk.io.read.file_reader import file_reader
from wrtdk.io.sim.sim_simulator import sim_simulator
from wrtdk.parser.msg.wrtmsg import udp_wrapper
from wrtdk.parser.imu.microstrain import lord3dm_cv5
from wrtdk.parser.nmea.nmea import gpgga
from wrtdk.parser.tfmag.qmag import qmag
from wrtdk.parser.tfmag.mfam import mx3g
from wrtdk.parser.vmag.pni import pni
from wrtdk.parser.alt.hcsr04 import hcsr04
from wrtdk.parser.alt.lidar_lite_v3 import lidar_lite_v3

class tbsim0_reader(file_reader):
    '''
    classdocs
    '''
    
    GGA_PATTERN = r'$G.GGA'
    TALKER_ID_LEN = len(GGA_PATTERN)
    GPS = (b'$GPS' + bytes([0,0])).decode()
    WRAPPER_COLS = 4

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__()
        
    def _read(self,filename):
        ''' reads the file and returns the data in a dictionary '''
        
        # intialize all parsers
        msgparser = sim_simulator()# get individual messages 
        wrapper = udp_wrapper()# udp wrapper parser
        imu = lord3dm_cv5()# Lord microstrain imu parser
        gga = gpgga()# nmea gga parser
        q = qmag()# qmag parser
        mfam = mx3g()
        vmag = pni()# pni parser
        sonar = hcsr04()# acoustic range finder parser
        lidar = lidar_lite_v3()# lidar parser
        
        # read the file 
        msgparser.read(filename)# read
        
        # cycle through all messages and count message types and system ids
        _len = msgparser.length()# get number of messages
        k = 0# counter for the messages
        msgs,data={},{}# intialize dictionaries to hold sensor count/length and data
        while k < _len:
            msg,_ = msgparser.getNext()# get the next message
            wrapper.parse(msg[0:wrapper.LENGTH])# parse the wrapper
            if not wrapper.hasErrored():# check to make sure no errors occurred
                s = str(wrapper.getSystemID())# get the system id key
                if s not in msgs.keys(): msgs[s] = {}# add dict to key is new msg type
                if wrapper.getType() == self.GPS:
                    if not self._gga(msg[wrapper.getMsgStart():wrapper.getMsgStart()+self.TALKER_ID_LEN]):
                        k += 1
                        continue
                if wrapper.getType() not in msgs[s].keys():# new sensor msg if new
                    msgs[s][wrapper.getType()] = {'count':0,'length':0}# count for filling array and length
                msgs[s][wrapper.getType()]['length'] += 1# increment the row number
            k += 1# increment the msg number
            
        #print(msgs)# uncomment to see how the messages count
        
        # initialze the output data array
        for system in msgs.keys():# cycle through all keys
            data[system] = {}# intialze key as a dict
            for sensor in msgs[system]:# cycel through all sensors
                # get the number of columsn depending on sensor type
                if sensor == '$SYSTM': cols = self.WRAPPER_COLS
                elif sensor == '$LIDAR': cols = self.WRAPPER_COLS+2
                elif sensor == '$TFMAG': cols = self.WRAPPER_COLS+2
                elif sensor == '$IMUX1': cols = self.WRAPPER_COLS+3
                elif sensor == '$VCMAG': cols = self.WRAPPER_COLS+4
                elif sensor == '$SONAR': cols = self.WRAPPER_COLS+2
                elif sensor == '$MFAM1': cols = self.WRAPPER_COLS+4
                elif sensor == '$MFAM2': cols = self.WRAPPER_COLS+4
                elif sensor == self.GPS: cols = self.WRAPPER_COLS+11
                else: cols = self.WRAPPER_COLS# default for unknown message
                
                # initialze numpy array to proper length and width
                data[system][sensor] = np.zeros([msgs[system][sensor]['length'],cols])
        
        # fill the arrays with sensor data
        k,row = 0,0
        while k < _len:# cycle through all messages
            msg,_ = msgparser.getNext()# get the next message
            wrapper.parse(msg[0:wrapper.LENGTH])# parse the header
            skey = str(wrapper.getSystemID())# find the system id key
            row = msgs[skey][wrapper.getType()]['count']# get the row number
            if row >= msgs[skey][wrapper.getType()]['length']: 
                k += 1
                print(msg)
                continue# skip if out of range
            
            # parse all the messages based on the sensor types
            if wrapper.getType() == '$TFMAG':# QMAG messages
                q.parse(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
                if not q.hasErrored():
                    d =  q.getData()
                    data[skey][wrapper.getType()][row,:] = self._hdr(wrapper) + [d[0][0],d[1][0]]
            elif wrapper.getType() == '$MFAM1':
                mfam.parse(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
                if not mfam.hasErrored():
                    d = mfam.getData()
                    data[skey][wrapper.getType()][row,:] = self._hdr(wrapper) + [d[0],d[6],d[1],d[7]]
            elif wrapper.getType() == '$MFAM2':
                mfam.parse(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
                if not mfam.hasErrored():
                    d = mfam.getData()
                    data[skey][wrapper.getType()][row,:] = self._hdr(wrapper) + [d[0],d[6],d[1],d[7]]
            elif wrapper.getType() == '$IMUX1':# Lord IMU
                imu.parse(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
                if not imu.hasErrored():
                    data[skey][wrapper.getType()][row,:] = self._hdr(wrapper) + imu.getData()[0:3]
            elif wrapper.getType() == '$VCMAG':# PNI vector mag
                vmag.parse(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
                if not vmag.hasErrored():
                    data[skey][wrapper.getType()][row,:] = self._hdr(wrapper) + vmag.getData()
            elif wrapper.getType() == '$SONAR':# acoustic range finder
                sonar.parse(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
                if not sonar.hasErrored():
                    data[skey][wrapper.getType()][row,:] = self._hdr(wrapper) + sonar.getData()
            elif wrapper.getType() == '$LIDAR':# lidar lite v3
                lidar.parse(msg[wrapper.getMsgStart():wrapper.getMsgEnd()])
                if not lidar.hasErrored():
                    data[skey][wrapper.getType()][row,:] = self._hdr(wrapper) + lidar.getData()[6::]
            elif wrapper.getType() == self.GPS:# all gps messages
                m = msg[wrapper.getMsgStart():wrapper.getMsgEnd()]
                if self._gga(m[0:self.TALKER_ID_LEN]):# all gpgga messages
                    gga.parse(m)
                    if not gga.hasErrored():
                        d = gga.getData()
                        data[skey][wrapper.getType()][row,:] = self._hdr(wrapper) + [d[0],
                                                                                     d[2],
                                                                                     d[3],
                                                                                     d[5],
                                                                                     d[6],
                                                                                     d[7],
                                                                                     d[8],
                                                                                     d[9],
                                                                                     ord(d[10]),
                                                                                     d[11],
                                                                                     d[12]]
                else:
                    k += 1
                    continue
            
            msgs[skey][wrapper.getType()]['count'] += 1# incremenent the row number
            k += 1# increment the msg number
            
        # reorder time to be chronological
        for skey in data.keys():# for each system id
            for hkey in data[skey].keys():# for each sensor 
                order = np.argsort(data[skey][hkey][:,0])# get order
                data[skey][hkey] = data[skey][hkey][order,:]# rearrange
        
        # return the data dict
        return data
        
    def _hdr(self,w):
        ''' gets the header info into a list '''
        return [w.getMCU(),w.getSensorID(),w.getFiducial(),self._status(w.getStatus())]
    
    def _gga(self,string):
        return string == b'$GPGGA' or string == b'$GNGGA'
            
    def _status(self,status):
        ''' correlates and integer with a status '''
        if status == 'AG': return 1
        elif status == 'ND': return 0
        elif status == 'MA': return 2
        elif status == 'ES': return 3
        elif status == 'L0': return 4
        elif status == 'L1': return 5
        elif status == 'L2': return 6
        elif status == 'L3': return 7
        elif status == 'L4': return 8
        elif status == 'L5': return 9
        elif status == 'L6': return 10
        elif status == 'L7': return 11
        elif status == 'L8': return 12
        elif status== 'l9': return 13
        else: return -1
            
        
if __name__ == '__main__':
    r = tbsim0_reader()
    data = r.read(r'C:\Users\reynolds\Documents\1704_madunit\data\20190308_enfield01\log00001.log')
    print(data)
    for skey in data.keys():
        print(skey)
        for hkey in data[skey].keys():
            print(hkey,data['4001']['$GPS\x00\x00'][:,0])
    
            


'''
Created on Nov 19, 2019

@author: reynolds
'''
import os, sys, struct
from wrtdk.parser.parser import parser

class emv1(parser):
    '''
    classdocs
    '''
    
    TX_MASK = 0b11100000
    RX_MASK = 0b00011100
    AX_MASK = 0b00000011
    COUNT_TO_MV = .1562
    PREAMBLE = b't'

    def __init__(self):
        '''
        Constructor
        '''
        super().__init__()
        self._len = self._minus1()
        self._bins = self._minus1()
        self._set = False
        self._fmt = ''
        
    def is_set(self):
        return self._set
        
    def reset(self):
        self._tx = self._minus1()
        self._rx = self._minus1()
        self._ax = self._minus1()
        self._ct = self._minus1()
        if self.is_set():
            for i in range(len(self._tg)):
                self._tg[i] = self._nan()
                self._mv[i] = self._nan()
            
    def set_bins(self,bins):
        self._bins = bins
        self._len = 1+1+1+self._bins*2
        self._tg = self._arr(self._bins)
        self._mv = self._arr(self._bins)
        self._set = True
        self._fmt = '>cBB%dhx'%self._bins
        
    def get_length(self):
        return self._len
        
    def get_bins(self):
        return self._bins
        
    def parse(self,msg):
        self.reset()
        
        try:
            if not self.is_set(): raise Exception('EMFormatException','Bin length is not set yet')
            m = struct.unpack(self._fmt,msg)
            
            if m[0] != self.PREAMBLE: raise Exception('EMFormatException','Message is not the proper format')
            
            self._tx = (m[1] & self.TX_MASK) >> 5
            self._rx = (m[1] & self.RX_MASK) >> 2
            self._ax = m[1] & self.AX_MASK
            self._ct = m[2]
            
            for i in range(self._bins):
                self._tg[i] = m[3+i]
                self._mv[i] = m[3+i] * self.COUNT_TO_MV
                
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))    
        
    def getData(self):
        ''' returns the data from the em message if set
        0) Tx
        1) Rx
        2) Ax
        3) Count
        4-end) millivolt bin data 
        '''
        if not self.is_set(): return None
        return [self._tx,self._rx,self._ax,self._ct]+self._mv


'''
Created on Aug 17, 2018

@author: reynolds
'''
from wrtdk.parser.parser import parser
from wrtdk.data.buffer.buffer import searcher
import os, sys
from wrtdk.parser.msg.wrtmsg import udp_wrapper

class qmag(parser):
    ''' parser for the qmag madunit messages '''
    
    def __init__(self):
        ''' constructor '''
        super().__init__()# inherit superclass
        self.reset()# initlaize properties
        # define constants
        self.NT_2_COUNT = 6009.342147
        self.COUNT_2_NT = 1.0/self.NT_2_COUNT
        
    def reset(self):
        ''' resets the data fields '''
        self._tf = []#self._nan()
        self._strength = []#-1
        
    def get(self,tf=0,strength=-1):
        ''' returns the raw bytes of the message '''
        return b'!%d@%d'%(round(tf*self.NT_2_COUNT),strength)
        
    def parse(self,msg):
        ''' parses a bytes array '''
        self.reset()
        
        try:
            string = msg.decode()
            string = string.split('@')
            
            # insert -1 if no strength found
            if len(string) < 2: string.insert(1,'-1')
                           
            self._tf.append(int(string[0][1:].rstrip('\x00'))*self.COUNT_2_NT)
            self._strength.append(int(string[1].rstrip('\x00')))
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
    def getData(self):
        ''' returns the data 
        0) list of total field in nT
        1) list of signal strength '''
        return self._tf,self._strength

class qmagasciibinary(qmag):
    ''' a parser for the nuwc qmag messages '''

    def __init__(self):
        ''' Constructor '''
        super().__init__()# inherit superclass
        self.reset()# initialize properties
        
        # define searcher for !
        self.search = searcher(b'!',w=0)
        
    def reset(self):
        ''' resets the properties '''
        self._tf = []
        self._strength = []
        
    def parse(self,msg):
        ''' parses John-Mikes stupid mixed ascii binary 
        with no delimiters message type '''
        self.reset()
        
        try:
            l = self.search.search(msg)
            l.append(len(msg))#mlen)
            
            # process indices
            d = [j-i for i, j in zip(l[:-1], l[1:])]# find the difference
            dd = [ j for (i,j) in zip(d,range(len(d))) if i <= 8 ]# find all differences too small
            d = [e for i,e in enumerate(l) if i not in dd]# remove any small differences
            
            for i in range(len(d)-1):
                string = msg[d[i]:d[i+1]-9].decode()
                string = string.split('@')
                
                # insert -1 if no strength found
                if len(string) < 2: string.insert(1,'-1')
                                
                self._tf.append(int(string[0][1:])*self.COUNT_2_NT)
                self._strength.append(int(string[1].strip()))
        except Exception as e:
            self._error = True
            exc_type, _, exc_tb = sys.exc_info()
            fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
            print('%s:%s in %s at %d. MSG:%s'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno,msg))
            
def test_qmagfile():
    q = qmag()
    filename = r'C:\Users\reynolds\Documents\1704_madunit\data\20180523_tethers\pre-deckcheckstatic_2018_05_23_1041.dat'
    msg = get_msg(filename)
    print('msg:%s' % msg)
    q.parse(msg)
    print(q.getData())
    
def test_qmagasciibinary():
    q = qmagasciibinary()
    filename = r'C:\Users\reynolds\Documents\1711_nuwc\data\20180732_deckcheck\20180731_deckcheckS01.dat'
    msg = get_msg(filename)
    print('msg:%s' % msg)
    q.parse(msg)
    print(q.getData())
    
def get_msg(filename):
    with open(filename,'rb') as f:
        idlen = 6
        msg = f.read(idlen)
        pos = 6
        
        while msg != b'$TFMAG' and pos < 1000:
            msg = msg[1::] + f.read(1)
            pos = pos + 1
            
        wrapper = udp_wrapper()
            
        msg = msg + f.read(36-6)
        pos = pos + 36 - 6
        wrapper.parse(msg)
        
        payload = f.read(wrapper.getLength())
        pos = pos + 1
        return payload
            
if __name__ == '__main__':
    test_qmagfile()
    print()
    test_qmagasciibinary()
    


'''
Created on Dec 17, 2019

@author: Reynolds
'''

import os

class file_reader(object):
    '''
    classdocs
    '''
    
    def read(self,filename):
        if not os.path.exists(filename):
            print('%s does not exist'%filename)
            return {}
        
        return self._read(filename)
        
    def _read(self,filename):
        pass


'''
Created on Mar 22, 2019

@author: reynolds
'''

import struct
from wrtdk.comms.comms import SerialPort

class e32_ebyte_lora(object):
    '''
    A class for interacting with the ebyte E32 long range radios
    '''

    def __init__(self,port=None):
        '''
        Constructor.
        
        port: Serial port information
        '''
        self.port = port
        
    def get_parameters(self):
        '''
        returns the current operation parameters as a byte array
        '''
        msg = struct.pack('BBB',193,193,193)
        port.write(msg)
        param,_ = port.readLine()
        for p in param: print(p,end=' ')
        print()
        return param
    
    def get_version(self):
        '''
        returns the present version number as a byte array
        '''
        msg = struct.pack('BBB',195,195,195)
        port.write(msg)
        param,_ = port.readLine()
        return param
    
    def reconfigure(self,head=192,addh=0,addl=0,sped=26,chan=15,option=68):
        '''
        head=194 -- > don't save settings on power down
        sped=34: --> ttl uart: 19200
        '''
        msg = struct.pack('BBBBBB',head,addh,addl,sped,chan,option)
        for m in msg: print(m,end=' ')
        print()
        print('reconfig:',msg)
        self.port.write(msg)
    
    def reset(self):
        '''
        resets the radio to its initial settings
        '''
        msg = struct.pack('BBB',196,196,196)
        port.write(msg)
        param,_ = port.readLine()
        return param
        
if __name__ == '__main__':
    port = SerialPort('COM12',9600)
    lora = e32_ebyte_lora(port)
    print('parameters:',lora.get_parameters())
    print('version:',lora.get_version())
    lora.reconfigure(head=192,sped=34)
    port.close()
        


'''
Created on Feb 14, 2019

@author: reynolds
'''

import sys, os, time

from PyQt5.QtCore import pyqtSignal

from wrtdk.io.streamer.streamer import LoggingStreamer
from wrtdk.parser.msg.wrtmsg import wrtsys, udp_wrapper, rtinv_mag, rtcom_mag, rtdet_mag,\
    rttrk_mag, sw_mag
from wrtdk.parser.imu.bno055 import bno055
from wrtdk.parser.imu.microstrain import lord3dm_cv5
from wrtdk.parser.tfmag.mfam import mx3g
from wrtdk.parser.tfmag.qmag import qmag
from wrtdk.parser.nmea.nmea import gpgga, gprmc
from wrtdk.parser.alt.hcsr04 import hcsr04
from wrtdk.parser.vmag.pni import pni
from wrtdk.io.sim.sim_simulator import sim_simulator
from wrtdk.parser.alt.lidar_lite_v3 import lidar_lite_v3
from wrtdk.parser.vehicle.gprip import gprip

class simstreamer(LoggingStreamer):
    ''' a class to read read data for the WRT sim board and send data to the ui '''
    
    # pyqtSignal to send data to the gui
    new_gga = pyqtSignal(list,'QString',int,int)
    new_rmc = pyqtSignal(list,'QString',int,int)
    new_vhw = pyqtSignal(list,'QString',int,int)
    new_vtg = pyqtSignal(list,'QString',int,int)
    new_hdt = pyqtSignal(list,'QString',int,int)
    new_gprip = pyqtSignal(list,'QString',int,int)
    new_qmag = pyqtSignal(list,'QString',int,int)
    new_mfam = pyqtSignal(list,'QString',int,int)
    new_vmag = pyqtSignal(list,'QString',int,int)
    new_imu = pyqtSignal(list,'QString',int,int)
    new_sonar = pyqtSignal(list,'QString',int,int)
    new_lidar = pyqtSignal(list,'QString',int,int)
    new_sys = pyqtSignal(list,'QString',int,int)
    new_mcu = pyqtSignal(list,'QString',int,int)
    new_rtcom_mag = pyqtSignal(list,'QString',int,int)
    new_rtdet_mag = pyqtSignal(list,'QString',int,int)
    new_rtinv_mag = pyqtSignal(list,'QString',int,int)
    new_rttrk_mag = pyqtSignal(list,'QString',int,int)
    new_timeout = pyqtSignal()

    def __init__(self,port=None,debug=False,filename=None,dt=0.1):
        ''' Constructor '''
        super().__init__(debug=debug)# inherit super class
        
        # set the port
        self.port = port
        
        # set the parsers
        self.h = udp_wrapper()
        self.systm = wrtsys()
        self.imu = bno055()
        self.imux1 = lord3dm_cv5()
        self.mfamo = mx3g()
        self.qmago = qmag()
        self.gga = gpgga()
        self.rmc = gprmc()
        self.gprip = gprip()
        self.hcsr = hcsr04()
        self.lite_v3 = lidar_lite_v3()
        self.vmag = pni()
        self.rtcommag = rtcom_mag()
        self.rtdetmag = rtdet_mag()
        self.rtinvmag = rtinv_mag()
        self.rttrkmag = rttrk_mag()
        self.swmag = sw_mag()
        
        # set the simulator
        self.sim = None# initialize
        self.dt = dt# timing
        if self._debug and os.path.join(filename):
            self.sim = sim_simulator()
            # read file
            if not self.sim.read(filename):# handle an improper file read
                self.sim = None# clear simulator
                self._debug = False# exit debug mode if not read properly
        
    def run(self):
        ''' main loop for thread '''
        self._running = True # set to run if not simulating
        
        # read the port and send signals
        while self._running:
            # read in the message
            try:
                msg,addr = self.port.read(1024)
            except Exception as e:
                exc_type, _, exc_tb = sys.exc_info()
                fname = os.path.split(exc_tb.tb_frame.f_code.co_filename)[1]
                print('%s:%s in %s at %d'%(exc_type.__name__,str(e), fname, exc_tb.tb_lineno))
                self.new_timeout.emit()
                continue
            
            self._getmsg(msg,addr)
                
        # close the port if not is sim mode
        self.port.close()
            
    def _getmsg(self,msg,addr):
        # check for invalid messages
            if len(msg) < self.h.LENGTH: print('Invalid message received received from',addr,msg)
            
            # write the new unaltered message to the log
            if self.writer.isLogging(): self.writer.write(msg)
            
            # define the position in the buffer
            pos = 0
            
            while pos < len(msg):   
                # parse the first header
                self.h.parse(msg[pos:pos+self.h.LENGTH])
                
                # check to see if the parser has errored and continue if it has
                if self.h.hasErrored():
                    print('Error reading header:',self.h.hasErrored())
                    break# this message is corrupt, wait for the next
                
                # parse the message
                self.parse(self.h,msg[self.h.getMsgStart(pos):self.h.getMsgEnd(pos)])
                
                # increment pos
                pos += self.h.getMsgEnd()
        
    def parse(self,h,msg):
        ''' parses the incoming message '''
        
        msg_type = h.getType()
        if not msg_type.startswith('$RT'):# send if it's not realtime message
            self.new_mcu.emit([h.getTimestamp(),h.getMCU()],
                              h.getStatus(),
                              h.getSystemID(),
                              h.getSensorID())
        
        if msg_type.startswith('$GPGGA'):# deprecated
            # parse the gpgga message and send it to te UI thread
            self.gga.parse(msg.decode().strip())
            if not self.gga.hasErrored():
                self.new_gga.emit(self.gga.getData(),
                                    h.getStatus(),
                                    h.getSystemID(),
                                    h.getSensorID())
        elif msg_type.startswith('$GPRMC'):# deprecated
            # send the status to the UI
            self.rmc.parse(msg.decode().strip())
            if not self.rmc.hasErrored():
                self.new_rmc.emit(self.rmc.getData(),
                                  h.getStatus(),
                                  h.getSystemID(),
                                  h.getSensorID())
        elif msg_type.startswith('$GPVHW'):# deprecated
            # send the status to the UI
            self.new_vhw.emit([msg.decode().strip()],
                              h.getStatus(),
                              h.getSystemID(),
                              h.getSensorID())
        elif msg_type.startswith('$GPHDT'):# deprecated
            # send the status to the UI
            self.new_hdt.emit([msg.decode().strip()],
                              h.getStatus(),
                              h.getSystemID(),
                              h.getSensorID())
        elif msg_type.startswith('$GPVTG'):# deprecated
            # send the status to the UI
            self.new_vtg.emit([msg.decode().strip()],
                              h.getStatus(),
                              h.getSystemID(),
                              h.getSensorID())
        elif msg_type.startswith('$GPS'):
            if msg.decode().startswith('$GPGGA'):
                # parse the gpgga message and send it to te UI thread
                self.gga.parse(msg.decode().strip())
                if not self.gga.hasErrored():
                    self.new_gga.emit(self.gga.getData(),
                                      h.getStatus(),
                                      h.getSystemID(),
                                      h.getSensorID())
            elif msg.decode().startswith('$GPRMC'):
                self.rmc.parse(msg.decode().strip())
                if not self.rmc.hasErrored():
                    self.new_rmc.emit(self.rmc.getData(),
                                      h.getStatus(),
                                      h.getSystemID(),
                                      h.getSensorID())
            elif msg.decode().startswith('$GPVHW'):
                self.new_vhw.emit([msg.decode().strip()],
                                  h.getStatus(),
                                  h.getSystemID(),
                                  h.getSensorID())
            elif msg.decode().startswith('$GPHDT'):
                self.new_hdt.emit([msg.decode().strip()],
                                  h.getStatus(),
                                  h.getSystemID(),
                                  h.getSensorID())
            elif msg.decode().startswith('$GPVTG'):
                self.new_vtg.emit([msg.decode().strip()],
                                  h.getStatus(),
                                  h.getSystemID(),
                                  h.getSensorID())
        elif msg_type.startswith('$GPRIP'):
            # parse and send the gprip message along
            self.gprip.parse(msg)
            if not self.gprip.hasErrored():
                self.new_gprip.emit(self.gprip.getData(),
                                    h.getStatus(),
                                    h.getSystemID(),
                                    h.getSensorID())
        elif msg_type.startswith('$TFMAG'):
            # parse the qmag message and send it to the UI thread
            self.qmago.parse(msg)
            if not self.qmago.hasErrored():
                _tf,_strength = self.qmago.getData()
                        
                # send every message to the UI
                for i in range(len(_tf)):
                    self.new_qmag.emit([_tf[i],_strength[i]],
                                       h.getStatus(),
                                       h.getSystemID(),
                                       h.getSensorID())
        elif (msg_type.startswith("$MFAM1")):
            # parse the mfam message and send it to the UI thread
            self.mfamo.parse(msg)                
            if not self.mfamo.hasErrored():
                self.new_mfam.emit([0] + self.mfamo.getData(),
                                    h.getStatus(),
                                    h.getSystemID(),
                                    h.getSensorID())
        elif msg_type.startswith("$MFAM2"):
            # parse the mfam message and send it to the UI thread
            self.mfamo.parse(msg)
            if not self.mfamo.hasErrored():
                self.new_mfam.emit([1] + self.mfamo.getData(),
                                    h.getStatus(),
                                    h.getSystemID(),
                                    h.getSensorID())
        elif msg_type.startswith('$VCMAG'):
            # parse the vector mag message and send it to the UI thread
            self.vmag.parse(msg)
            if not self.vmag.hasErrored():
                self.new_vmag.emit(self.vmag.getData(),
                                   h.getStatus(),
                                   h.getSystemID(),
                                   h.getSensorID())
        elif msg_type.startswith('$IMUXX'):
            # parse the imu message and send it to the UI thread
            self.imu.parse(msg)
            if not self.imu.hasErrored():
                self.new_imu.emit(self.imu.getData(),
                                  h.getStatus(),
                                  h.getSystemID(),
                                  h.getSensorID())
        elif msg_type.startswith('$IMUX1'):
            # parse the imu message and send it to the UI thread
            self.imux1.parse(msg)
            if not self.imux1.hasErrored():
                self.new_imu.emit(self.imux1.getData(),
                                  h.getStatus(),
                                  h.getSystemID(),
                                  h.getSensorID())
        elif msg_type.startswith('$SONAR'):
            # parse the sonar message and send to the UI thread
            self.hcsr.parse(msg)
            if not self.hcsr.hasErrored():
                self.new_sonar.emit(self.hcsr.getData(),
                                    h.getStatus(),
                                    h.getSystemID(),
                                    h.getSensorID())
        elif msg_type.startswith('$LIDAR'):
            self.lite_v3.parse(msg)
            if not self.lite_v3.hasErrored():
                self.new_lidar.emit(self.lite_v3.getData(),
                                      h.getStatus(),
                                      h.getSystemID(),
                                      h.getSensorID())
        elif msg_type.startswith('$RTCOM'):
            # parse the real time compensation method
            self.rtcommag.parse(msg)
            if not self.rtcommag.hasErrored():
                self.new_rtcom_mag.emit(self.rtcommag.getData(),
                                        h.getStatus(),
                                        h.getSystemID(),
                                        h.getSensorID())
        elif msg_type.startswith('$RTDET'):
            # parse the real time detection method
            self.rtdetmag.parse(msg)
            if not self.rtdetmag.hasErrored():
                self.new_rtdet_mag.emit(self.rtdetmag.getData(),
                                        h.getStatus(),
                                        h.getSystemID(),
                                        h.getSensorID())
        elif msg_type.startswith('$RTINV'):
            # parse the real time inversion method
            self.rtinvmag.parse(msg)
            if not self.rtinvmag.hasErrored():
                self.new_rtinv_mag.emit(self.rtinvmag.getData(),
                                        h.getStatus(),
                                        h.getSystemID(),
                                        h.getSensorID())
        elif msg_type.startswith('$RTTRK'):
            # parse the real time track message
            self.rttrkmag.parse(msg)
            if not self.rttrkmag.hasErrored():
                self.new_rttrk_mag.emit(self.rttrkmag.getData(),
                                        h.getStatus(),
                                        h.getSystemID(),
                                        0)# make sure the ui knows the message is from rttrk message
        elif (msg_type.startswith('$SWIDL') or 
              msg_type.startswith('$SWHUN') or
              msg_type.startswith('$SWSTP') or
              msg_type.startswith('$SWCOR') or 
              msg_type.startswith('$SWDUN') or 
              msg_type.startswith('$SWABT')):
            if msg_type.startswith('$SWABT'): sen = 1
            elif msg_type.startswith('$SWIDL'): sen = 2
            elif msg_type.startswith('$SWHUN'): sen = 3
            elif msg_type.startswith('$SWSTP'): sen = 4 
            elif msg_type.startswith('$SWCOR'): sen = 5
            elif msg_type.startswith('$SWDUN'): sen = 6
            else: sen = -1
            self.swmag.parse(msg)
            if not self.swmag.hasErrored():
                self.new_rttrk_mag.emit(self.swmag.getData(),
                                        h.getStatus(),
                                        h.getSystemID(),
                                        sen)
        elif msg_type.startswith('$SYSTM'):
            # parse the system message and send it to the UI thread
            self.systm.parse(msg)    
            if not self.systm.hasErrored():
                self.new_sys.emit(self.systm.getData(),
                                  h.getStatus(),
                                  h.getSystemID(),
                                  h.getSensorID())
        else: print('UnknownMessageType:%s %s'% (msg_type,msg))


'''
Created on Aug 17, 2018

@author: reynolds
'''

from wrtdk.comms.comms import SerialPort

def set_auto(port,baud=9600,close=True,verb=True):
    if type(port) == type(str): port = SerialPort(port,baud)
    elif type(port) != type(SerialPort()):
        print('Please make sure that the port entry is a str or SerialPort object')
        return
    else:
        print('Please make sure that the port entry is a str or SerialPort object')
        return
    
    # write commands to fluxgate
    port.write(b'0L\r')
    if verb: print(port.readLine())
    port.write(b'0wc01b5A\r')
    if verb: print(port.readLine())
    port.write(b'0L\r')
    if verb: print(port.readLine())
    port.write(b'0wc08b10\r')
    if verb: print(port.readLine())
    
    # close the port
    if close: port.close()
    
def set_poll(port,baud=9600,close=True,verb=True):
    if type(port) == type(str): port = SerialPort(port,baud)
    elif type(port) != type(SerialPort()):
        print('Please make sure that the port entry is a str or SerialPort object')
        return
    else:
        print('Please make sure that the port entry is a str or SerialPort object')
        return
    
    # write commands to fluxgate
    port.write(b'0l\r')
    if verb: print(port.readLine())
    port.write(b'0wv1\r')
    if verb: print(port.readLine())
    port.write(b'0L\r')
    if verb: print(port.readLine())
    port.write(b'0wc01b00\r')
    if verb: print(port.readLine())
    
    #close the port
    if close: port.close()
    
def poll(port,baud=9600,close=True,verb=True):
    if type(port) == type(str): port = SerialPort(port,baud)
    elif type(port) != type(SerialPort()):
        print('Please make sure that the port entry is a str or SerialPort object')
        return
    else:
        print('Please make sure that the port entry is a str or SerialPort object')
        return
    
    # write commands to fluxgate
    port.write(b'0SD')
    print(port.readLine())
    
    #close the port
    if close: port.close()

class ap1540(object):
    ''' a parser for the ap150 fluxgate '''

    def __init__(self):
        ''' Constructor '''
        super().__init__()
        
        # initialzie properties
        self.reset()
        
        # define constants
        self.COUNT_2_NT = 1000/75
        
    def reset(self):
        ''' resets the properties '''
        self._bx = self._nan()
        self._by = self._nan()
        self._bx = self._nan()
        
    def parse(self,msg):
        ''' parses the ap1540 message '''
        print(msg)


'''
Created on Feb 14, 2019

@author: reynolds
'''

import sys

from PyQt5.QtWidgets import QApplication,QMainWindow, QPushButton, QGridLayout, QWidget
from PyQt5.QtCore import pyqtSlot
from wrtdk.instrument.fieldfox.N9918A import N9918A
from wrtdk.comms.comms import SerialPort
from wrtdk.io.streamer.vnastreamer import fieldfox_streamer

class test(QMainWindow):
    '''
    classdocs
    '''

    def __init__(self):
        ''' Constructor '''
        QMainWindow.__init__(self)
        
        main = QWidget(self)
        btn = QPushButton('Connect')
        btn.clicked.connect(self._connect)
        log = QPushButton('Log')
        log.clicked.connect(self._log)
        
        l = QGridLayout()
        l.addWidget(btn)
        l.addWidget(log)
        
        main.setLayout(l)
        self.setFocus()
        self.setCentralWidget(main)
        
        self._port = None
        self._thread = None
        
    def _connect(self):
        if self._port is None:
            self._port = []
            self._port.append(N9918A(dict(verb=True,
                                          addr='128.128.204.202',
                                          trans='OFF',
                                          form='POL',
                                          timeout=2000,
                                          chunk_size=102400,
                                          dform='REAL,64',
                                          np=401,
                                          spwr=-1.6)))
            self._port.append(SerialPort('COM8',9600,timeout=0))
            self._thread = fieldfox_streamer(self._port)
            self._thread.new_ascan.connect(self._new_scan)
            self._thread.new_d.connect(self._new_dist)
            self._thread.start()
        else:
            self._thread.stop()
            self._thread.wait()
            self._thread = None
            self._port = None
            
    def _log(self):
        if self._thread.isRunning():
            print(self._thread.isLogging())
            if self._thread.isLogging():
                self._thread.stopLog()
            else:
                self._thread.startLog('test.csv')
                
    @pyqtSlot(list,str,int,int)
    def _new_scan(self,data,status,system,sensor):
        ''' reports the data from the fieldfox '''
        print('ASCAN','n:',data[2],status,system,sensor)
        print(data[3],data[4])
    
    @pyqtSlot(list,str,int,int)
    def _new_dist(self,data,status,system,sensor):
        ''' reports the data from the motor '''
        print(data,status,system,sensor)
        
if __name__ == '__main__':
    app = QApplication(sys.argv)
    aw = test()
    aw.show()
    app.exec_()